package com.sun.corba.se.impl.interceptors;

import org.omg.CORBA.BAD_INV_ORDER;
import com.sun.corba.se.spi.servicecontext.UnknownServiceContext;
import com.sun.corba.se.impl.encoding.EncapsOutputStream;
import org.omg.IOP.ServiceContextHelper;
import com.sun.corba.se.spi.ior.iiop.GIOPVersion;
import sun.corba.OutputStreamFactory;
import java.util.HashMap;
import com.sun.corba.se.impl.orbutil.ORBUtility;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.ParameterMode;
import org.omg.CORBA.NVList;
import org.omg.CORBA.UNKNOWN;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import org.omg.CORBA.CompletionStatus;
import java.io.IOException;
import org.omg.CORBA.UserException;
import org.omg.CORBA.portable.InputStream;
import com.sun.corba.se.impl.util.RepositoryId;
import sun.corba.SharedSecrets;
import org.omg.CORBA.portable.ApplicationException;
import org.omg.IOP.ServiceContext;
import org.omg.PortableInterceptor.InvalidSlot;
import org.omg.CORBA.Object;
import org.omg.CORBA.Any;
import org.omg.CORBA.TypeCode;
import org.omg.Dynamic.Parameter;
import com.sun.corba.se.spi.ior.IOR;
import org.omg.PortableInterceptor.ForwardRequest;
import com.sun.corba.se.spi.servicecontext.ServiceContexts;
import com.sun.corba.se.spi.legacy.connection.Connection;
import com.sun.corba.se.impl.logging.OMGSystemException;
import com.sun.corba.se.impl.logging.InterceptorsSystemException;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.spi.legacy.interceptor.RequestInfoExt;
import org.omg.PortableInterceptor.RequestInfo;
import org.omg.CORBA.LocalObject;

public abstract class RequestInfoImpl extends LocalObject implements RequestInfo, RequestInfoExt
{
    protected ORB myORB;
    protected InterceptorsSystemException wrapper;
    protected OMGSystemException stdWrapper;
    protected int flowStackIndex;
    protected int startingPointCall;
    protected int intermediatePointCall;
    protected int endingPointCall;
    protected short replyStatus;
    protected static final short UNINITIALIZED = -1;
    protected int currentExecutionPoint;
    protected static final int EXECUTION_POINT_STARTING = 0;
    protected static final int EXECUTION_POINT_INTERMEDIATE = 1;
    protected static final int EXECUTION_POINT_ENDING = 2;
    protected boolean alreadyExecuted;
    protected Connection connection;
    protected ServiceContexts serviceContexts;
    protected ForwardRequest forwardRequest;
    protected IOR forwardRequestIOR;
    protected SlotTable slotTable;
    protected Exception exception;
    protected static final int MID_REQUEST_ID = 0;
    protected static final int MID_OPERATION = 1;
    protected static final int MID_ARGUMENTS = 2;
    protected static final int MID_EXCEPTIONS = 3;
    protected static final int MID_CONTEXTS = 4;
    protected static final int MID_OPERATION_CONTEXT = 5;
    protected static final int MID_RESULT = 6;
    protected static final int MID_RESPONSE_EXPECTED = 7;
    protected static final int MID_SYNC_SCOPE = 8;
    protected static final int MID_REPLY_STATUS = 9;
    protected static final int MID_FORWARD_REFERENCE = 10;
    protected static final int MID_GET_SLOT = 11;
    protected static final int MID_GET_REQUEST_SERVICE_CONTEXT = 12;
    protected static final int MID_GET_REPLY_SERVICE_CONTEXT = 13;
    protected static final int MID_RI_LAST = 13;
    
    void reset() {
        this.flowStackIndex = 0;
        this.startingPointCall = 0;
        this.intermediatePointCall = 0;
        this.endingPointCall = 0;
        this.setReplyStatus((short)(-1));
        this.currentExecutionPoint = 0;
        this.alreadyExecuted = false;
        this.connection = null;
        this.serviceContexts = null;
        this.forwardRequest = null;
        this.forwardRequestIOR = null;
        this.exception = null;
    }
    
    public RequestInfoImpl(ORB myORB) {
        this.flowStackIndex = 0;
        this.replyStatus = -1;
        this.myORB = myORB;
        this.wrapper = InterceptorsSystemException.get(myORB, "rpc.protocol");
        this.stdWrapper = OMGSystemException.get(myORB, "rpc.protocol");
        this.slotTable = ((PICurrent)(PICurrent)myORB.getPIHandler().getPICurrent()).getSlotTable();
    }
    
    public abstract int request_id();
    
    public abstract String operation();
    
    public abstract Parameter[] arguments();
    
    public abstract TypeCode[] exceptions();
    
    public abstract String[] contexts();
    
    public abstract String[] operation_context();
    
    public abstract Any result();
    
    public abstract boolean response_expected();
    
    public short sync_scope() {
        this.checkAccess(8);
        return 1;
    }
    
    public short reply_status() {
        this.checkAccess(9);
        return this.replyStatus;
    }
    
    public abstract Object forward_reference();
    
    public Any get_slot(int n) throws InvalidSlot {
        return this.slotTable.get_slot(n);
    }
    
    public abstract ServiceContext get_request_service_context(int p0);
    
    public abstract ServiceContext get_reply_service_context(int p0);
    
    public Connection connection() {
        return this.connection;
    }
    
    private void insertApplicationException(ApplicationException ex, Any any) throws UNKNOWN {
        try {
            Method method = SharedSecrets.getJavaCorbaAccess().loadClass(RepositoryId.cache.getId(ex.getId()).getClassName() + "Helper").getMethod("read", new Class[] { InputStream.class });
            InputStream inputStream = ex.getInputStream();
            inputStream.mark(0);
            UserException ex2 = null;
            try {
                ex2 = (UserException)method.invoke(null, new java.lang.Object[] { inputStream });
            }
            finally {
                try {
                    inputStream.reset();
                }
                catch (IOException ex3) {
                    throw this.wrapper.markAndResetFailed(ex3);
                }
            }
            this.insertUserException(ex2, any);
        }
        catch (ClassNotFoundException ex4) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex4);
        }
        catch (NoSuchMethodException ex5) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex5);
        }
        catch (SecurityException ex6) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex6);
        }
        catch (IllegalAccessException ex7) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex7);
        }
        catch (IllegalArgumentException ex8) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex8);
        }
        catch (InvocationTargetException ex9) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex9);
        }
    }
    
    private void insertUserException(UserException ex, Any any) throws UNKNOWN {
        try {
            if (ex != null) {
                Class<java.lang.Object> class1 = ex.getClass();
                SharedSecrets.getJavaCorbaAccess().loadClass(class1.getName() + "Helper").getMethod("insert", new Class[] { Any.class, class1 }).invoke(null, new java.lang.Object[] { any, ex });
            }
        }
        catch (ClassNotFoundException ex2) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex2);
        }
        catch (NoSuchMethodException ex3) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex3);
        }
        catch (SecurityException ex4) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex4);
        }
        catch (IllegalAccessException ex5) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex5);
        }
        catch (IllegalArgumentException ex6) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex6);
        }
        catch (InvocationTargetException ex7) {
            throw this.stdWrapper.unknownUserException(CompletionStatus.COMPLETED_MAYBE, ex7);
        }
    }
    
    protected Parameter[] nvListToParameterArray(NVList list) {
        int count = list.count();
        Parameter[] array = new Parameter[count];
        try {
            for (int i = 0; i < count; ++i) {
                array[i] = new Parameter();
                NamedValue item = list.item(i);
                array[i].argument = item.value();
                array[i].mode = ParameterMode.from_int(item.flags() - 1);
            }
        }
        catch (Exception ex) {
            throw this.wrapper.exceptionInArguments(ex);
        }
        return array;
    }
    
    protected Any exceptionToAny(Exception ex) {
        Any create_any = this.myORB.create_any();
        if (ex == null) {
            throw this.wrapper.exceptionWasNull2();
        }
        if ((ex instanceof SystemException)) {
            ORBUtility.insertSystemException((SystemException)ex, create_any);
        }
        else if ((ex instanceof ApplicationException)) {
            try {
                this.insertApplicationException((ApplicationException)ex, create_any);
            }
            catch (UNKNOWN unknown2) {
                ORBUtility.insertSystemException(unknown2, create_any);
            }
        }
        else if ((ex instanceof UserException)) {
            try {
                this.insertUserException((UserException)ex, create_any);
            }
            catch (UNKNOWN unknown) {
                ORBUtility.insertSystemException(unknown, create_any);
            }
        }
        return create_any;
    }
    
    protected ServiceContext getServiceContext(HashMap hashMap, ServiceContexts serviceContexts, int value) {
        Integer n = new Integer(value);
        ServiceContext read = (ServiceContext)hashMap.get(n);
        if (read == null) {
            com.sun.corba.se.spi.servicecontext.ServiceContext value2 = serviceContexts.get(value);
            if (value2 == null) {
                throw this.stdWrapper.invalidServiceContextId();
            }
            EncapsOutputStream encapsOutputStream = OutputStreamFactory.newEncapsOutputStream(this.myORB);
            value2.write(encapsOutputStream, GIOPVersion.V1_2);
            read = ServiceContextHelper.read(encapsOutputStream.create_input_stream());
            hashMap.put(n, read);
        }
        return read;
    }
    
    protected void addServiceContext(HashMap hashMap, ServiceContexts serviceContexts, ServiceContext value, boolean b) {
        EncapsOutputStream encapsOutputStream = OutputStreamFactory.newEncapsOutputStream(this.myORB);
        ServiceContextHelper.write(encapsOutputStream, value);
        InputStream create_input_stream = encapsOutputStream.create_input_stream();
        UnknownServiceContext unknownServiceContext = new UnknownServiceContext(create_input_stream.read_long(), (org.omg.CORBA_2_3.portable.InputStream)create_input_stream);
        int id = unknownServiceContext.getId();
        if (serviceContexts.get(id) != null) {
            if (b) {
                serviceContexts.delete(id);
            }
            else {
                throw this.stdWrapper.serviceContextAddFailed(new Integer(id));
            }
        }
        serviceContexts.put(unknownServiceContext);
        hashMap.put(new Integer(id), value);
    }
    
    protected void setFlowStackIndex(int flowStackIndex) {
        this.flowStackIndex = flowStackIndex;
    }
    
    protected int getFlowStackIndex() {
        return this.flowStackIndex;
    }
    
    protected void setEndingPointCall(int endingPointCall) {
        this.endingPointCall = endingPointCall;
    }
    
    protected int getEndingPointCall() {
        return this.endingPointCall;
    }
    
    protected void setIntermediatePointCall(int intermediatePointCall) {
        this.intermediatePointCall = intermediatePointCall;
    }
    
    protected int getIntermediatePointCall() {
        return this.intermediatePointCall;
    }
    
    protected void setStartingPointCall(int startingPointCall) {
        this.startingPointCall = startingPointCall;
    }
    
    protected int getStartingPointCall() {
        return this.startingPointCall;
    }
    
    protected boolean getAlreadyExecuted() {
        return this.alreadyExecuted;
    }
    
    protected void setAlreadyExecuted(boolean alreadyExecuted) {
        this.alreadyExecuted = alreadyExecuted;
    }
    
    protected void setReplyStatus(short replyStatus) {
        this.replyStatus = replyStatus;
    }
    
    protected short getReplyStatus() {
        return this.replyStatus;
    }
    
    protected void setForwardRequest(ForwardRequest forwardRequest) {
        this.forwardRequest = forwardRequest;
        this.forwardRequestIOR = null;
    }
    
    protected void setForwardRequest(IOR forwardRequestIOR) {
        this.forwardRequestIOR = forwardRequestIOR;
        this.forwardRequest = null;
    }
    
    protected ForwardRequest getForwardRequestException() {
        if (this.forwardRequest == null && this.forwardRequestIOR != null) {
            this.forwardRequest = new ForwardRequest(this.iorToObject(this.forwardRequestIOR));
        }
        return this.forwardRequest;
    }
    
    protected IOR getForwardRequestIOR() {
        if (this.forwardRequestIOR == null && this.forwardRequest != null) {
            this.forwardRequestIOR = ORBUtility.getIOR(this.forwardRequest.forward);
        }
        return this.forwardRequestIOR;
    }
    
    protected void setException(Exception exception) {
        this.exception = exception;
    }
    
    Exception getException() {
        return this.exception;
    }
    
    protected void setCurrentExecutionPoint(int currentExecutionPoint) {
        this.currentExecutionPoint = currentExecutionPoint;
    }
    
    protected abstract void checkAccess(int p0) throws BAD_INV_ORDER;
    
    void setSlotTable(SlotTable slotTable) {
        this.slotTable = slotTable;
    }
    
    protected Object iorToObject(IOR ior) {
        return ORBUtility.makeObjectReference(ior);
    }
}
